/*
 * Copyright (c) 2004 Anil Madhavapeddy <anil@recoil.org>
 * Copyright (c) 2004 Intel Research Cambridge
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $Id: log.c,v 1.6 2004/09/07 15:52:04 avsm Exp $
 */

#include "log.h"

static int log_level = L_NONE;
static char *string_of_log_level(int);
static pthread_mutex_t log_mutex;
const char default_id[] = "main";

void
logmsg(const char *id, int class, const char *fmt, ...) {
    va_list ap;
    if (class > log_level)
        return;
    if (!id)
        id = default_id;
    va_start(ap, fmt);
    pthread_mutex_lock(&log_mutex);
    fprintf(stderr, "[%s] [%s] ", id, string_of_log_level(class));
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, "\n");
    pthread_mutex_unlock(&log_mutex);
    fflush(stderr);
    va_end(ap);
}

/* Must be called at start of program before first log */
void
init_log(void) {
    pthread_mutex_init(&log_mutex, NULL);
}

void
set_log_level(int class) {
    if (log_level < L_NONE || log_level > L_DEBUG)
        errx(1, "invalid log level %d\n", class);
    log_level = class;
}

static char *
string_of_log_level(int level) {
    switch (level) {
    case L_NONE:
        return "none";
    case L_CRITICAL:
        return "critical";
    case L_INFO:
        return "info";
    case L_DEBUG:
        return "debug";
    default:
        break;
    }
    errx(1, "switch_of_log_level, unknown level %d\n", level);
}
